from copy import deepcopy
from datetime import datetime, timezone

data = {
    "unixfrom": "From 0101017629c51320-4f67ae0a-8056-4cd7-b526-e47da4bbe7b7-000000@ses.pypi.org  Thu Dec  3 18:07:47 2020",
    "headers": {
        "return-path": [
            "<0101017629c51320-4f67ae0a-8056-4cd7-b526-e47da4bbe7b7-000000@ses.pypi.org>",
        ],
        "x-original-to": [
            "recipient@redacted.nil",
        ],
        "delivered-to": [
            "recipient@redacted.nil",
        ],
        "x-greylist": [
            "delayed 316 seconds by postgrey-1.36 at firefly; Thu, 03 Dec 2020 18:07:47 UTC",
        ],
        "authentication-results": [
            "mail.redacted.nil;\tdkim=pass (1024-bit key; unprotected) header.d=pypi.org header.i=@pypi.org header.a=rsa-sha256 header.s=sy4yaueax2wxeqvbwtfafsfpuli4zspd header.b=RopfsgK8;\tdkim=pass (1024-bit key; unprotected) header.d=amazonses.com header.i=@amazonses.com header.a=rsa-sha256 header.s=hsbnp7p3ensaochzwyq5wwmceodymuwv header.b=PvoEs17L;\tdkim-atps=neutral",
        ],
        "received": [
            "from a27-10.smtp-out.us-west-2.amazonses.com (a27-10.smtp-out.us-west-2.amazonses.com [54.240.27.10])\tby mail.redacted.nil (Postfix) with ESMTPS id 5EAAE13B030\tfor <recipient@redacted.nil>; Thu,  3 Dec 2020 18:07:47 +0000 (UTC)",
        ],
        "dkim-signature": [
            "v=1; a=rsa-sha256; q=dns/txt; c=relaxed/simple;\ts=sy4yaueax2wxeqvbwtfafsfpuli4zspd; d=pypi.org; t=1607018550;\th=Subject:From:To:MIME-Version:Content-Type:Message-ID:Date;\tbh=3tSjr/p8U98Y6VPb/lXnFWQX4OvAj20xcNdTK2v2o6k=;\tb=RopfsgK8yiLka57yfeNmWJKg4i6ToHif2+7H8gRketwz+0QfsjU/mRcGJUkVOGVQ\tv2wlDwboRC2G39QbX53YYyo4yQBPXokRImT8xRkDpHomqd0OUS0odtE8THHack8EEaE\t3gfX+t80fHbILQiNtAyjXg60IlivJemzFEs93tks=",
            "v=1; a=rsa-sha256; q=dns/txt; c=relaxed/simple;\ts=hsbnp7p3ensaochzwyq5wwmceodymuwv; d=amazonses.com; t=1607018550;\th=Subject:From:To:MIME-Version:Content-Type:Message-ID:Date:Feedback-ID;\tbh=3tSjr/p8U98Y6VPb/lXnFWQX4OvAj20xcNdTK2v2o6k=;\tb=PvoEs17LVcXnf9YScJ9IPD4pVhs1mzl4YYGv2fCT0oG7xdKodFSlZJ7Z0haY1APE\tT86HnVq7bzjsCWL9TQJWAi+LrHBlIRLRL22k6YNDuwYJOa9dkKFxbiaqwEIe+bzxuEX\tPpdWM/1xxHuQ9a11INQRSH8it0sZXwSTBTqRyaAU=",
        ],
        "subject": "[PyPI] Collaborator added notification",
        "from": [
            {
                "display_name": "PyPI",
                "address": "noreply@pypi.org",
            },
        ],
        "to": [
            {
                "display_name": "jwodder",
                "address": "recipient@redacted.nil",
            },
        ],
        "content-type": {
            "content_type": "multipart/alternative",
            "params": {},
        },
        "message-id": "<0101017629c51320-4f67ae0a-8056-4cd7-b526-e47da4bbe7b7-000000@us-west-2.amazonses.com>",
        "date": datetime(2020, 12, 3, 18, 2, 30, tzinfo=timezone.utc),
        "x-ses-outgoing": [
            "2020.12.03-54.240.27.10",
        ],
        "feedback-id": [
            "1.us-west-2.ZRFc7KXPaRh5E01FOWpg1s1VfcS7E0CO2O03ZZnJB3U=:AmazonSES",
        ],
        "status": ["RO"],
        "content-length": ["1431"],
        "lines": ["65"],
    },
    "preamble": None,
    "content": [
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "text/plain",
                    "params": {},
                },
            },
            "preamble": None,
            "content": "\nA new collaborator has been added to a project you own on PyPI:\n\n  Username: REDACTED\n  Role: Owner\n  Collaborator for: REDACTED\n  Added by: jwodder\n\nIf this was a mistake, you can email admin@pypi.org to communicate with the PyPI administrators.\n\n\n--\n\nYou are receiving this because you are an owner of this project.\n",
            "epilogue": None,
        },
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "text/html",
                    "params": {},
                },
            },
            "preamble": None,
            "content": "<!DOCTYPE html>\n<html>\n  <head>\n    <meta charset=\"utf-8\">\n    <meta name=\"viewport\" content=\"width=device-width\">\n\n    </head>\n  <body>\n    \n<p>\n  A new collaborator has been added to a project you own on PyPI:\n\n  </p>\n<ul style=\"list-style-type:none\">\n    <li>\n<strong>Username</strong>: REDACTED</li>\n    <li>\n<strong>Role</strong>: Owner</li>\n    <li>\n<strong>Collaborator for</strong>: REDACTED</li>\n    <li>\n<strong>Added by</strong>: jwodder</li>\n  </ul>\n\n\n<p>If this was a mistake, you can email <a href=\"mailto:admin@pypi.org\">admin@pypi.org</a> to communicate with the PyPI administrators.</p>\n\n\n    \n    <p style=\"color:#666; font-size:small\">\n        —<br>\n        \nYou are receiving this because you are an owner of this project.\n\n    </p>\n    \n  </body>\n</html>\n",
            "epilogue": None,
        },
    ],
    "epilogue": "",
}

data_all = deepcopy(data)
data_all["headers"]["mime-version"] = "1.0"
data_all["headers"]["content-type"]["params"]["boundary"] = "===============0747615486913937503=="
data_all["content"][0]["headers"]["content-type"]["params"]["charset"] = "utf-8"
data_all["content"][0]["headers"]["content-transfer-encoding"] = "7bit"
data_all["content"][1]["headers"]["content-type"]["params"]["charset"] = "utf-8"
data_all["content"][1]["headers"]["content-transfer-encoding"] = "quoted-printable"
data_all["content"][1]["headers"]["mime-version"] = "1.0"



from copy import deepcopy
import email
from email import policy
from email.headerregistry import Address
from email.message import EmailMessage
from email.utils import make_msgid
from importlib import import_module
import mailbox
from operator import attrgetter
from pathlib import Path
import pytest
from email2dict import email2dict

DATA_DIR = Path(__file__).with_name("data")
EMAIL_DIR = DATA_DIR / "emails"
MBOX_DIR = DATA_DIR / "mboxes"

def test_simple():
    BODY = (
        "Oh my beloved!\n"
        "\n"
        "Wilt thou dine with me on the morrow?\n"
        "\n"
        "We're having hot pockets.\n"
        "\n"
        "Love, Me\n"
    )
    msg = EmailMessage()
    msg["Subject"] = "Meet me"
    msg["To"] = "my.beloved@love.love"
    msg["From"] = "me@here.qq"
    msg.set_content(BODY)
    DICT = {
        "unixfrom": None,
        "headers": {
            "subject": "Meet me",
            "to": [
                {
                    "display_name": "",
                    "address": "my.beloved@love.love",
                },
            ],
            "from": [
                {
                    "display_name": "",
                    "address": "me@here.qq",
                },
            ],
            "content-type": {
                "content_type": "text/plain",
                "params": {},
            },
        },
        "preamble": None,
        "content": BODY,
        "epilogue": None,
    }
    assert email2dict(msg) == DICT
    DICT_ALL = deepcopy(DICT)
    DICT_ALL["headers"]["content-type"]["params"]["charset"] = "utf-8"
    DICT_ALL["headers"]["content-transfer-encoding"] = "7bit"
    DICT_ALL["headers"]["mime-version"] = "1.0"
    assert email2dict(msg, include_all=True) == DICT_ALL

def test_text_html_attachment():
    # Adapted from <https://docs.python.org/3/library/email.examples.html>
    msg = EmailMessage()
    msg["Subject"] = "Ayons asperges pour le déjeuner"
    msg["From"] = Address("Pepé Le Pew", "pepe", "example.com")
    msg["To"] = (Address("Penelope Pussycat", "penelope", "example.com"),
                 Address("Fabrette Pussycat", "fabrette", "example.com"))
    TEXT = (
        "Salut!\n"
        "\n"
        "Cela ressemble à un excellent recipie[1] déjeuner.\n"
        "\n"
        "[1] http://www.yummly.com/recipe/Roasted-Asparagus-Epicurious-203718\n"
        "\n"
        "--Pepé\n"
    )
    msg.set_content(TEXT)
    asparagus_cid = make_msgid()
    HTML = (
        "<html>\n"
        "  <head></head>\n"
        "  <body>\n"
        "    <p>Salut!</p>\n"
        "    <p>Cela ressemble à un excellent\n"
        "        <a href=\"http://www.yummly.com/recipe/Roasted-Asparagus-"
        "Epicurious-203718\">\n"
        "            recipie\n"
        "        </a> déjeuner.\n"
        "    </p>\n"
        f"    <img src=\"cid:{asparagus_cid[1:-1]}\" />\n"
        "  </body>\n"
        "</html>\n"
    )
    msg.add_alternative(HTML, subtype='html')
    IMG = b'\1\2\3\4\5'
    msg.get_payload()[1].add_related(IMG, 'image', 'png', cid=asparagus_cid)
    DICT = {
        "unixfrom": None,
        "headers": {
            "subject": "Ayons asperges pour le déjeuner",
            "from": [
                {
                    "display_name": "Pepé Le Pew",
                    "address": "pepe@example.com",
                },
            ],
            "to": [
                {
                    "display_name": "Penelope Pussycat",
                    "address": "penelope@example.com",
                },
                {
                    "display_name": "Fabrette Pussycat",
                    "address": "fabrette@example.com",
                },
            ],
            "content-type": {
                "content_type": "multipart/alternative",
                "params": {},
            },
        },
        "preamble": None,
        "content": [
            {
                "unixfrom": None,
                "headers": {
                    "content-type": {
                        "content_type": "text/plain",
                        "params": {},
                    },
                },
                "preamble": None,
                "content": TEXT,
                "epilogue": None,
            },
            {
                "unixfrom": None,
                "headers": {
                    "content-type": {
                        "content_type": "multipart/related",
                        "params": {},
                    },
                },
                "preamble": None,
                "content": [
                    {
                        "unixfrom": None,
                        "headers": {
                            "content-type": {
                                "content_type": "text/html",
                                "params": {},
                            },
                        },
                        "preamble": None,
                        "content": HTML,
                        "epilogue": None,
                    },
                    {
                        "unixfrom": None,
                        "headers": {
                            "content-type": {
                                "content_type": "image/png",
                                "params": {},
                            },
                            "content-disposition": {
                                "disposition": "inline",
                                "params": {},
                            },
                            "content-id": [asparagus_cid],
                        },
                        "preamble": None,
                        "content": IMG,
                        "epilogue": None,
                    },
                ],
                "epilogue": None,
            },
        ],
        "epilogue": None,
    }
    assert email2dict(msg) == DICT
    DICT_ALL = deepcopy(DICT)
    DICT_ALL["headers"]["mime-version"] = "1.0"
    DICT_ALL["content"][0]["headers"]["content-transfer-encoding"] = "8bit"
    DICT_ALL["content"][0]["headers"]["content-type"]["params"]["charset"] = "utf-8"
    DICT_ALL["content"][1]["headers"]["mime-version"] = "1.0"
    DICT_ALL["content"][1]["content"][0]["headers"]["content-transfer-encoding"] = "quoted-printable"
    DICT_ALL["content"][1]["content"][0]["headers"]["content-type"]["params"]["charset"] = "utf-8"
    DICT_ALL["content"][1]["content"][1]["headers"]["mime-version"] = "1.0"
    DICT_ALL["content"][1]["content"][1]["headers"]["content-transfer-encoding"] = "base64"
    assert email2dict(msg, include_all=True) == DICT_ALL

@pytest.mark.parametrize("eml", EMAIL_DIR.glob("*.eml"), ids=attrgetter("name"))
def test_actual_emails(eml, monkeypatch):
    with eml.open("rb") as fp:
        msg = email.message_from_binary_file(fp, policy=policy.default)
    monkeypatch.syspath_prepend(EMAIL_DIR)
    module = import_module(eml.stem)
    assert email2dict(msg) == module.data
    assert email2dict(msg, include_all=True) == module.data_all

@pytest.mark.parametrize("mbox", MBOX_DIR.glob("*.mbox"), ids=attrgetter("name"))
def test_actual_mboxes(mbox, monkeypatch):
    box = mailbox.mbox(mbox)
    box.lock()
    msg, = box
    box.close()
    monkeypatch.syspath_prepend(MBOX_DIR)
    module = import_module(mbox.stem)
    assert email2dict(msg) == module.data
    assert email2dict(msg, include_all=True) == module.data_all

def test_text_image_mixed():
    PNG = bytes.fromhex(
        '89 50 4e 47 0d 0a 1a 0a  00 00 00 0d 49 48 44 52'
        '00 00 00 10 00 00 00 10  08 06 00 00 00 1f f3 ff'
        '61 00 00 00 06 62 4b 47  44 00 ff 00 ff 00 ff a0'
        'bd a7 93 00 00 00 09 70  48 59 73 00 00 00 48 00'
        '00 00 48 00 46 c9 6b 3e  00 00 00 09 76 70 41 67'
        '00 00 00 10 00 00 00 10  00 5c c6 ad c3 00 00 00'
        '5b 49 44 41 54 38 cb c5  92 51 0a c0 30 08 43 7d'
        'b2 fb 5f 39 fb 12 da 61  a9 c3 8e f9 a7 98 98 48'
        '90 64 9d f2 16 da cc ae  b1 01 26 39 92 d8 11 10'
        '16 9e e0 8c 64 dc 89 b9  67 80 ca e5 f3 3f a8 5c'
        'cd 76 52 05 e1 b5 42 ea  1d f0 91 1f b4 09 78 13'
        'e5 52 0e 00 ad 42 f5 bf  85 4f 14 dc 46 b3 32 11'
        '6c b1 43 99 00 00 00 00  49 45 4e 44 ae 42 60 82'
    )
    body = EmailMessage()
    body.set_content("This is part 1.\n")
    image = EmailMessage()
    image.set_content(
        PNG,
        "image",
        "png",
        disposition = "inline",
        filename = "ternary.png",
    )
    msg = EmailMessage()
    msg["Subject"] = "Text and an image"
    msg.make_mixed()
    msg.attach(body)
    msg.attach(image)
    DICT = {
        "unixfrom": None,
        "headers": {
            "subject": "Text and an image",
            "content-type": {
                "content_type": "multipart/mixed",
                "params": {},
            },
        },
        "preamble": None,
        "content": [
            {
                "unixfrom": None,
                "headers": {
                    "content-type": {
                        "content_type": "text/plain",
                        "params": {},
                    },
                },
                "preamble": None,
                "content": "This is part 1.\n",
                "epilogue": None,
            },
            {
                "unixfrom": None,
                "headers": {
                    "content-type": {
                        "content_type": "image/png",
                        "params": {},
                    },
                    "content-disposition": {
                        "disposition": "inline",
                        "params": {
                            "filename": "ternary.png",
                        },
                    },
                },
                "preamble": None,
                "content": PNG,
                "epilogue": None,
            },
        ],
        "epilogue": None,
    }
    assert email2dict(msg) == DICT
    DICT_ALL = deepcopy(DICT)
    DICT_ALL["content"][0]["headers"]["mime-version"] = "1.0"
    DICT_ALL["content"][0]["headers"]["content-transfer-encoding"] = "7bit"
    DICT_ALL["content"][0]["headers"]["content-type"]["params"]["charset"] = "utf-8"
    DICT_ALL["content"][1]["headers"]["mime-version"] = "1.0"
    DICT_ALL["content"][1]["headers"]["content-transfer-encoding"] = "base64"
    assert email2dict(msg, include_all=True) == DICT_ALL



from datetime import datetime, timedelta, timezone

data = {
    "unixfrom": None,
    "headers": {
        "resent-from": [
            {
                "display_name": "Mary Smith",
                "address": "mary@example.net",
            },
        ],
        "resent-sender": [
            {
                "display_name": "Michael Jones",
                "address": "mjones@machine.example",
            },
        ],
        "resent-to": [
            {
                "display_name": "Jane Brown",
                "address": "j-brown@other.example",
            },
        ],
        "resent-date": [
            datetime(1997, 11, 24, 14, 22, 1, tzinfo=timezone(timedelta(hours=-8))),
        ],
        "resent-message-id": [
            "<78910@example.net>",
        ],
        "from": [
            {
                "display_name": "John Doe",
                "address": "jdoe@machine.example",
            },
        ],
        "to": [
            {
                "display_name": "Mary Smith",
                "address": "mary@example.net",
            },
        ],
        "subject": "Saying Hello",
        "date": datetime(1997, 11, 21, 9, 55, 6, tzinfo=timezone(timedelta(hours=-6))),
        "message-id": "<1234@local.machine.example>",
    },
    "preamble": None,
    "content": "This is a message just to say hello.\nSo, \"Hello\".\n\n-- \nAdapted from RFC 5322, section A.3.\n",
    "epilogue": None,
}

data_all = data



from copy import deepcopy
from datetime import datetime, timedelta, timezone

data = {
    "unixfrom": None,
    "headers": {
        "delivered-to": ["recipient@redacted.nil"],
        "received": [
            "by 2002:ac0:a2c2:0:0:0:0:0 with SMTP id 60csp4719765imt;        Mon, 28 Dec 2020 07:07:19 -0800 (PST)",
            "from mail.python.org (mail.python.org. [188.166.95.178])        by mx.google.com with ESMTPS id by27si18939218edb.276.2020.12.28.07.07.19        for <recipient@redacted.nil>        (version=TLS1_3 cipher=TLS_AES_256_GCM_SHA384 bits=256/256);        Mon, 28 Dec 2020 07:07:19 -0800 (PST)",
            "from mail.python.org (mail1.ams1.psf.io [127.0.0.1])\tby mail.python.org (Postfix) with ESMTP id 4D4LWC07XmzpFYf\tfor <recipient@redacted.nil>; Mon, 28 Dec 2020 10:07:19 -0500 (EST)",
            "from mail.python.org (mail1.ams1.psf.io [127.0.0.1])\tby mail.python.org (Postfix) with ESMTP id 4D1nTl11NLzpFQk\tfor <distutils-sig@python.org>; Thu, 24 Dec 2020 06:12:11 -0500 (EST)",
            "from mail1.ams1.psf.io (HELO mail.python.org) (127.0.0.1)  by mail.python.org with SMTP; 24 Dec 2020 06:12:11 -0500",
            "from mail-ej1-x62d.google.com (mail-ej1-x62d.google.com [IPv6:2a00:1450:4864:20::62d])\t(using TLSv1.3 with cipher TLS_AES_256_GCM_SHA384 (256/256 bits)\t key-exchange X25519 server-signature RSA-PSS (2048 bits) server-digest SHA256)\t(No client certificate requested)\tby mail.python.org (Postfix) with ESMTPS\tfor <distutils-sig@python.org>; Thu, 24 Dec 2020 06:12:10 -0500 (EST)",
            "by mail-ej1-x62d.google.com with SMTP id w1so2888903ejf.11        for <distutils-sig@python.org>; Thu, 24 Dec 2020 03:12:10 -0800 (PST)",
        ],
        "x-google-smtp-source": [
            "ABdhPJzA/Taij6vwL6SyA+Fl8im5Y84Os0+Hk7RimG4HmDdtm6X5GLYDTv2QQr/H9u50/8w93C4/",
        ],
        "x-received": [
            "by 2002:a17:906:878d:: with SMTP id za13mr41554256ejb.395.1609168039403;        Mon, 28 Dec 2020 07:07:19 -0800 (PST)",
            "by 2002:a17:907:2108:: with SMTP id qn8mr27369460ejb.127.1608808330208; Thu, 24 Dec 2020 03:12:10 -0800 (PST)",
        ],
        "arc-seal": [
            "i=1; a=rsa-sha256; t=1609168039; cv=none;        d=google.com; s=arc-20160816;        b=umN621V3wTbn5iGxVRov5MOa9vXGY3b/KQgYrJEIVWs3vRpkw/iWuQcsbcG6h6iHWz         ch3yAHtHhvYktFV1iR5e+NlEueY6V5QT4yW1impStJWqW4CN3bEOgvHAXJ2KwNOSrr3K         cFW+1R39QqYn1P9zv5HpLvP2isI1/WZQXxc7UsJ1JaMN2146ioFKtzVb06n9AGAmHAtj         OTpUQQnF1AemCmT0Ge2Q//bhErXn6xUcuwEN5enAW9NYPCgyGw7UZ4YADUjBJ3fopNUX         hi7KnWVMiZP2XwOVbsIIbE9TahhmCa4WRkPA2vY+Ne8UbKzpO4Y2xMIMxv2Mc2IUUc9q         WsSQ==",
        ],
        "arc-message-signature": [
            "i=1; a=rsa-sha256; c=relaxed/relaxed; d=google.com; s=arc-20160816;        h=list-unsubscribe:list-subscribe:list-post:list-help:list-archive         :archived-at:list-id:subject:precedence:message-id-hash:to         :message-id:date:from:mime-version:dkim-signature:dkim-signature;        bh=HqMw7IM1Rimz/7z4Mk+sE6QIRK23hm2DEzjUCvPEmAI=;        b=vd7Sx2kOx6PZKpQB/eouARKfHSYwGH5Pj241HhAmdSMMDS3CGkjir+lvuhtQ3kjo7b         FNROpjqtAzl4eJ3F3EMYo+j8Tt/NLHLnwiwpbqJVdHO+suVbKrQ9oidOPj6QWrgavk7n         V9De8kMDlBlpiUhp8K0IWA4VBa3qcfKIZHqukQovqqyx1LOqTg2NwcHemma7d2ky/Ue/         lClTsGlBD7O4gsoFtyDRzdvmCya/VB9l8J6A3qAwHCbabzRFN8hBHdYjD1EjUo+d2fCy         LHhZIw6iPILaUS+q6V1vFC6Y1dHbErpUpNQygCp61DQTs4KBxnZGam6u/MtUW2gvA6Jv         /q0A==",
        ],
        "arc-authentication-results": [
            "i=1; mx.google.com;       dkim=pass header.i=@python.org header.s=200901 header.b=aBfU1fxY;       dkim=neutral (body hash did not verify) header.i=@gmail.com header.s=20161025 header.b=amXVzdXG;       spf=pass (google.com: domain of distutils-sig-bounces+recipient=redacted.nil@python.org designates 188.166.95.178 as permitted sender) smtp.mailfrom=\"distutils-sig-bounces+recipient=redacted.nil@python.org\";       dmarc=fail (p=NONE sp=QUARANTINE dis=NONE) header.from=gmail.com",
        ],
        "return-path": [
            "<distutils-sig-bounces+recipient=redacted.nil@python.org>",
        ],
        "received-spf": [
            "pass (google.com: domain of distutils-sig-bounces+recipient=redacted.nil@python.org designates 188.166.95.178 as permitted sender) client-ip=188.166.95.178;",
        ],
        "authentication-results": [
            "mx.google.com;       dkim=pass header.i=@python.org header.s=200901 header.b=aBfU1fxY;       dkim=neutral (body hash did not verify) header.i=@gmail.com header.s=20161025 header.b=amXVzdXG;       spf=pass (google.com: domain of distutils-sig-bounces+recipient=redacted.nil@python.org designates 188.166.95.178 as permitted sender) smtp.mailfrom=\"distutils-sig-bounces+recipient=redacted.nil@python.org\";       dmarc=fail (p=NONE sp=QUARANTINE dis=NONE) header.from=gmail.com",
            "mail.python.org; dkim=pass\treason=\"2048-bit key; unprotected key\"\theader.d=gmail.com header.i=@gmail.com header.b=amXVzdXG;\tdkim-adsp=pass; dkim-atps=neutral",
        ],
        "dkim-signature": [
            "v=1; a=rsa-sha256; c=relaxed/simple; d=python.org; s=200901;\tt=1609168039; bh=HqMw7IM1Rimz/7z4Mk+sE6QIRK23hm2DEzjUCvPEmAI=;\th=From:Date:To:Subject:List-Id:List-Archive:List-Help:List-Post:\t List-Subscribe:List-Unsubscribe:From;\tb=aBfU1fxYMEarNvHY5G2dwqBwz+oYnqa2Qzpn1oYoPF8b0ANMQ6uXThvuHC5hAMQvo\t hkpxvtNSmUloOrHR1xNIUc8LNnteN6n8Gp+9fRhNAFL9I5JCeensUHnK7yhb6K4Qo1\t 0uH2RI2zNtgkMzfhj5dd22mKaCeURTVuQ268S7OA=",
            "v=1; a=rsa-sha256; c=relaxed/relaxed;        d=gmail.com; s=20161025;        h=mime-version:from:date:message-id:subject:to;        bh=gA+0zvQbFGXK6SfGpDQYT7ZyNP+hEupURUMiqONEWZE=;        b=amXVzdXG3Fs3j2JL1GN6HZe3tFlHNSCiSN1oY5+ohCVbVkHTn7+ZyJ+jSUw/Hpwc7U         SvAmNjJ+nALjUxL7rOvZH3D4VpwuA9ngdW0evz2WdKXUxPd9/61iGMQfxrYVxp/huLmw         BfNqe7YvT+x5INw+capdy0EzTfJj/EfG4SQ2a0Tz7TxwDIr8YkQgUBIIxNZ//xVSzoD1         gtZRua7iEso6iLcV/r2CMYj3rNaMTFPLg4zlJt/u7PYmBOdBuK/4pwSsI8w8eI9bqms2         cfLPVV/RNeIzGVcKOlxJCOK+ctRCD/6E9ZXtSZFzq7WnKnKDBgpHBC40xGKLyPs3VHkX         vVHw==",
        ],
        "x-spam-status": [
            "OK 0.007",
        ],
        "x-spam-evidence": [
            "'*H*': 0.99; '*S*': 0.00; 'url-ip:151.101.0.223/32':    0.09; 'url-ip:151.101.128.223/32': 0.09; 'url-    ip:151.101.192.223/32': 0.09; 'url-ip:151.101.64.223/32': 0.09;    'notable': 0.16; 'upload': 0.16; 'url:latest': 0.16;    'url:project': 0.16; 'url:pypi': 0.16; 'option': 0.23; 'message-    id:@mail.gmail.com': 0.33; 'improvements': 0.33;    'received:google.com': 0.34; 'from:addr:gmail.com': 0.35;    'include': 0.38; 'to:addr:distutils-sig': 0.68",
        ],
        "x-google-dkim-signature": [
            "v=1; a=rsa-sha256; c=relaxed/relaxed;        d=1e100.net; s=20161025;        h=x-gm-message-state:mime-version:from:date:message-id:subject:to;        bh=gA+0zvQbFGXK6SfGpDQYT7ZyNP+hEupURUMiqONEWZE=;        b=Y/6X+bpP7SHCvrsZRpV6yeQ6l5FQPmnTuagQwZaVoCshovoOi0+FHDfRgp+dkaWV+K         JMRyLWJXEKOnjM1sawBnG2LxRFeVwqhxrajrXIHfctQzBs0Qe62Hd6ZTLLFKrYvKz5Rh         5nZgeRf2i0NqlJGcDlFUGB9Bs+7KF8+d6M7QFvj08H9M7roIBQWqdvMbUA0m8BUzzNVO         qvVIDB6nsWuhBWlR2X24e1CUrzb5E++X/2BuVWasL0ZoaAzqhKN+ikycPMeCLmCQ8kxg         J+Q+0ntj1aN5yeZ+1N4/uNz52Om2bh0RU/aJ7zYcfjYLMmXhZ7Jasb8+ramfsb34Vfwm         3ZOw==",
        ],
        "x-gm-message-state": [
            "AOAM531pZltQDlpsqrcuHPwXpusl/KUwJQiaT1WBiG3Q0iVXWBBPa62s\tSZ3xcR6VvMO9Z1Ezdn5ufVoKkFmVjfR1MNRr3iDpDl7t0nwpqg=="
        ],
        "from": [
            {
                "display_name": "Brian Rutledge",
                "address": "bhrutledge@gmail.com",
            }
        ],
        "date": datetime(2020, 12, 24, 6, 11, 59, tzinfo=timezone(timedelta(hours=-5))),
        "message-id": "<CAKQj8i4mZQGGtmLe+rHAZNBXNv2Hw6dg5cHtzQL6e71mnZkanA@mail.gmail.com>",
        "to": [
            {
                "display_name": "",
                "address": "distutils-sig@python.org",
            },
        ],
        "x-mailfrom": ["bhrutledge@gmail.com"],
        "x-mailman-rule-hits": ["member-moderation"],
        "x-mailman-rule-misses": ["dmarc-mitigation; no-senders; approved; emergency; loop; banned-address"],
        "message-id-hash": ["5FC6OKCCVOJZGRYTKAK7CA5RC2XVX24Z"],
        "x-message-id-hash": ["5FC6OKCCVOJZGRYTKAK7CA5RC2XVX24Z"],
        "x-mailman-approved-at": ["Mon, 28 Dec 2020 10:06:20 -0500"],
        "x-mailman-version": ["3.3.3b1"],
        "precedence": ["list"],
        "subject": "[Distutils] Twine 3.3.0 released",
        "list-id": ["\"Python packaging ecosystem SIG (PyPI/pip/twine/wheel/setuptools/distutils/etc)\" <distutils-sig.python.org>"],
        "archived-at": ["<https://mail.python.org/archives/list/distutils-sig@python.org/message/5FC6OKCCVOJZGRYTKAK7CA5RC2XVX24Z/>"],
        "list-archive": ["<https://mail.python.org/archives/list/distutils-sig@python.org/>"],
        "list-help": ["<mailto:distutils-sig-request@python.org?subject=help>"],
        "list-post": ["<mailto:distutils-sig@python.org>"],
        "list-subscribe": ["<mailto:distutils-sig-join@python.org>"],
        "list-unsubscribe": ["<mailto:distutils-sig-leave@python.org>"],
        "content-type": {
            "content_type": "multipart/mixed",
            "params": {},
        },
    },
    "preamble": None,
    "content": [
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "multipart/alternative",
                    "params": {},
                },
            },
            "preamble": None,
            "content": [
                {
                    "unixfrom": None,
                    "headers": {
                        "content-type": {
                            "content_type": "text/plain",
                            "params": {},
                        },
                    },
                    "preamble": None,
                    "content": "https://pypi.org/project/twine/3.3.0/\n\nChangelog (now via towncrier):\nhttps://twine.readthedocs.io/en/latest/changelog.html\n\nNotable improvements include more `twine upload --verbose` output, and a\n`--strict` option for `twine check`.\n",
                    "epilogue": None,
                },
                {
                    "unixfrom": None,
                    "headers": {
                        "content-type": {
                            "content_type": "text/html",
                            "params": {},
                        },
                    },
                    "preamble": None,
                    "content": "<div dir=\"ltr\"><div><a href=\"https://pypi.org/project/twine/3.3.0/\">https://pypi.org/project/twine/3.3.0/</a></div><div><br></div><div>Changelog (now via towncrier): <a href=\"https://twine.readthedocs.io/en/latest/changelog.html\">https://twine.readthedocs.io/en/latest/changelog.html</a></div><div><br></div><div>Notable improvements include more `twine upload --verbose` output, and a `--strict` option for `twine check`.</div></div>\n",
                    "epilogue": None,
                },
            ],
            "epilogue": "",
        },
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "text/plain",
                    "params": {},
                },
                "content-disposition": {
                    "disposition": "inline",
                    "params": {},
                },
            },
            "preamble": None,
            "content": "--\nDistutils-SIG mailing list -- distutils-sig@python.org\nTo unsubscribe send an email to distutils-sig-leave@python.org\nhttps://mail.python.org/mailman3/lists/distutils-sig.python.org/\nMessage archived at https://mail.python.org/archives/list/distutils-sig@python.org/message/5FC6OKCCVOJZGRYTKAK7CA5RC2XVX24Z/\n",
            "epilogue": None,
        },
    ],
    "epilogue": "",
}

data_all = deepcopy(data)
data_all["headers"]["mime-version"] = "1.0"
data_all["headers"]["content-type"]["params"]["boundary"] = "===============5457169869532207506=="
data_all["content"][0]["headers"]["content-type"]["params"]["boundary"] = "00000000000008a9d405b733e1f4"
data_all["content"][0]["content"][0]["headers"]["content-type"]["params"]["charset"] = "UTF-8"
data_all["content"][0]["content"][1]["headers"]["content-type"]["params"]["charset"] = "UTF-8"
data_all["content"][0]["content"][1]["headers"]["content-transfer-encoding"] = "quoted-printable"
data_all["content"][1]["headers"]["content-type"]["params"]["charset"] = "us-ascii"
data_all["content"][1]["headers"]["mime-version"] = "1.0"
data_all["content"][1]["headers"]["content-transfer-encoding"] = "7bit"



from copy import deepcopy

data = {
    "unixfrom": None,
    "headers": {
        "content-type": {
            "content_type": "multipart/related",
            "params": {
                "start": "<950120.aaCC@XIson.com>",
                "type": "Application/X-FixedRecord",
                "start-info": "-o ps",
            },
        },
        "comment": [
            "Taken from (errata'ed) RFC 2387, section 5.1",
        ],
    },
    "preamble": None,
    "content": [
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "application/x-fixedrecord",
                    "params": {},
                },
                "content-id": [
                    "<950120.aaCC@XIson.com>",
                ],
            },
            "preamble": None,
            "content": b"25\n10\n34\n10\n25\n21\n26\n10",
            "epilogue": None,
        },
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "application/octet-stream",
                    "params": {},
                },
                "content-description": [
                    "The fixed length records",
                ],
                "content-id": [
                    "<950120.aaCB@XIson.com>",
                ],
            },
            "preamble": None,
            "content": b"Old MacDonald had a farm\nE I E I O\nAnd on his farm he had some ducks\nE I E I O\nWith a quack quack here,\na quack quack there,\nevery where a quack quack\nE I E I O\n",
            "epilogue": None,
        },
    ],
    "epilogue": "",
}

data_all = deepcopy(data)
data_all["headers"]["content-type"]["params"]["boundary"] = "example-1"
data_all["content"][1]["headers"]["content-transfer-encoding"] = "base64"



data = {
    "unixfrom": None,
    "headers": {
        "from": [
            {
                "display_name": "Keith Moore",
                "address": "moore@cs.utk.edu",
            },
        ],
        "to": [
            {
                "display_name": "Keld Jørn Simonsen",
                "address": "keld@dkuug.dk",
            },
        ],
        "cc": [
            {
                "display_name": "André Pirard",
                "address": "PIRARD@vm1.ulg.ac.be",
            },
        ],
        "subject": "If you can read this you understand the example.",
    },
    "preamble": None,
    "content": "Taken from RFC 2047, section 8.\n",
    "epilogue": None,
}

data_all = data



"""
Convert EmailMessage objects to dicts

``email2dict`` converts Python ``Message`` & ``EmailMessage`` instances into
structured ``dict``\\s.  Need to examine a ``Message`` but find the builtin
Python API too fiddly?  Need to check that a ``Message`` has the content &
structure you expect?  Need to compare two ``Message`` instances for equality?
Need to pretty-print the structure of a ``Message``?  Then ``email2dict`` has
your back.

Visit <https://github.com/jwodder/email2dict> for more information.
"""

__version__      = '0.1.0'
__author__       = 'John Thorvald Wodder II'
__author_email__ = 'email2dict@varonathe.org'
__license__      = 'MIT'
__url__          = 'https://github.com/jwodder/email2dict'

from   datetime        import datetime
import email
from   email           import headerregistry as hr
from   email           import policy
from   email.generator import BytesGenerator
from   email.message   import EmailMessage, Message
import inspect
from   io              import BytesIO
from   mailbox         import MMDFMessage, mboxMessage
import sys
from   typing          import Any, Callable, Dict, List, Optional, \
                                TYPE_CHECKING, cast

if TYPE_CHECKING:
    if sys.version_info[:2] >= (3, 8):
        from typing import TypedDict
    else:
        from typing_extensions import TypedDict

    class MessageDict(TypedDict):
        unixfrom: Optional[str]
        headers: Dict[str, Any]
        preamble: Optional[str]
        content: Any
        epilogue: Optional[str]


__all__ = ["email2dict"]

def process_unique_string_header(ush: List[Any]) -> str:
    assert len(ush) == 1
    return str(ush[0])

def process_address(addr: hr.Address) -> Dict[str, str]:
    return {
        "display_name": addr.display_name,
        "address": addr.addr_spec,
    }

def process_addr_headers(ahs: List[Any]) -> List[dict]:
    data: List[dict] = []
    for h in ahs:
        assert isinstance(h, hr.AddressHeader)
        for g in h.groups:
            addrlist: List[dict]
            if g.display_name is not None:
                group = {"group": g.display_name, "addresses": []}
                data.append(group)
                addrlist = cast(List[dict], group["addresses"])
            else:
                addrlist = data
            addrlist.extend(map(process_address, g.addresses))
    return data

SKIPPED_CT_PARAMS = {
    "charset",
    "boundary",
}

def process_content_type_headers(cths: List[Any], include_all: bool = False) \
        -> Dict[str, Any]:
    assert len(cths) == 1
    return {
        "content_type": cths[0].content_type,
        "params": {
            k: v for k,v in cths[0].params.items()
                 if include_all or k not in SKIPPED_CT_PARAMS
        },
    }

def process_date_headers(dh: List[Any]) -> List[datetime]:
    data = []
    for h in dh:
        assert isinstance(h, hr.DateHeader)
        data.append(h.datetime)
    return data

def process_unique_date_header(dh: List[Any]) -> datetime:
    assert len(dh) == 1
    assert isinstance(dh[0], hr.UniqueDateHeader)
    return dh[0].datetime

def process_unique_single_addr_header(ah: List[Any]) -> Dict[str, str]:
    assert len(ah) == 1
    assert isinstance(ah[0], hr.UniqueSingleAddressHeader)
    return process_address(ah[0].address)

def process_single_addr_header(ah: List[Any]) -> List[Dict[str, str]]:
    data = []
    for h in ah:
        assert isinstance(h, hr.SingleAddressHeader)
        data.append(process_address(h.address))
    return data

def process_content_disposition_header(cdh: List[Any]) -> Dict[str, Any]:
    assert len(cdh) == 1
    assert isinstance(cdh[0], hr.ContentDispositionHeader)
    return {
        "disposition": cdh[0].content_disposition,
        "params": dict(cdh[0].params),
    }

def process_cte_header(cteh: List[Any]) -> str:
    assert len(cteh) == 1
    assert isinstance(cteh[0], hr.ContentTransferEncodingHeader)
    return cteh[0].cte  # TODO: When is this different from `str(cteh[0])`?

def process_mime_version_header(mvh: List[Any]) -> Optional[str]:
    assert len(mvh) == 1
    assert isinstance(mvh[0], hr.MIMEVersionHeader)
    return mvh[0].version

HEADER_PROCESSORS: Dict[str, Callable] = {
    "subject": process_unique_string_header,
    "message-id": process_unique_string_header,
    "from": process_addr_headers,
    "to": process_addr_headers,
    "cc": process_addr_headers,
    "bcc": process_addr_headers,
    "content-type": process_content_type_headers,
    "date": process_unique_date_header,
    "resent-date": process_date_headers,
    "orig-date": process_unique_date_header,
    "resent-to": process_addr_headers,
    "resent-cc": process_addr_headers,
    "resent-bcc": process_addr_headers,
    "resent-from": process_addr_headers,
    "reply-to": process_addr_headers,
    "sender": process_unique_single_addr_header,
    "resent-sender": process_single_addr_header,
    "content-disposition": process_content_disposition_header,
    "content-transfer-encoding": process_cte_header,
    "mime-version": process_mime_version_header,
}

SKIPPED_HEADERS = {
    "content-transfer-encoding",
    "mime-version",
}

def email2dict(msg: Message, include_all: bool = False) -> "MessageDict":
    if not isinstance(msg, EmailMessage):
        msg = message2email(msg)
    data: MessageDict = {
        "unixfrom": msg.get_unixfrom(),
        "headers": {},
        "preamble": None,
        "content": None,
        "epilogue": None,
    }
    for header in msg.keys():
        header = header.lower()
        if header in SKIPPED_HEADERS and not include_all:
            continue
        values = msg.get_all(header)
        if not values:
            continue
        try:
            processor = HEADER_PROCESSORS[header]
        except KeyError:
            v = list(map(str, values))
        else:
            kwargs = {}
            if takes_argument(processor, "include_all"):
                kwargs["include_all"] = include_all
            v = processor(values, **kwargs)
        data["headers"][header] = v
    data["preamble"] = msg.preamble
    if msg.get_content_maintype() == "message":
        # Some "message/*" subtypes (specifically, as of Python 3.9, rfc822 and
        # external-body, but we should try to be future-proof) have
        # get_content() return a Message, while others return bytes.
        content = msg.get_content()
        if isinstance(content, Message):
            data["content"] = email2dict(content, include_all=include_all)
        else:
            data["content"] = content
    elif msg.is_multipart():
        data["content"] = [
            email2dict(p, include_all=include_all) for p in msg.iter_parts()
        ]
    else:
        data["content"] = msg.get_content()
    data["epilogue"] = msg.epilogue
    return data

def message2email(msg: Message) -> EmailMessage:
    # Message.as_bytes() refolds long header lines (which can result in changes
    # in whitespace after reparsing) and doesn't give a way to change this, so
    # we need to use a BytesGenerator manually.
    fp = BytesIO()
    g = BytesGenerator(fp, mangle_from_=False, maxheaderlen=0)
    g.flatten(msg, unixfrom=msg.get_unixfrom() is not None)
    fp.seek(0)
    emsg = email.message_from_binary_file(fp, policy=policy.default)
    assert isinstance(emsg, EmailMessage)
    # MMDFMessage and mboxMessage make their "From " lines available though a
    # different method than normal Messages, so we have to copy it over
    # manually.
    if isinstance(msg, (MMDFMessage, mboxMessage)):
        emsg.set_unixfrom("From " + msg.get_from())
    return emsg

def takes_argument(callable_obj: Callable, argname: str) -> bool:
    sig = inspect.signature(callable_obj)
    for param in sig.parameters.values():
        if (
            param.kind in (param.POSITIONAL_OR_KEYWORD, param.KEYWORD_ONLY)
            and param.name == argname
        ):
            return True
        elif param.kind is param.VAR_KEYWORD:
            return True
    return False



from datetime import datetime, timedelta, timezone

data = {
    "unixfrom": None,
    "headers": {
        "from": [
            {
                "display_name": "Pete",
                "address": "pete@silly.example",
            },
        ],
        "to": [
            {
                "group": "A Group",
                "addresses": [
                    {
                        "display_name": "Ed Jones",
                        "address": "c@a.test",
                    },
                    {
                        "display_name": "",
                        "address": "joe@where.test",
                    },
                    {
                        "display_name": "John",
                        "address": "jdoe@one.test",
                    },
                ],
            },
        ],
        "cc": [
            {
                "group": "Undisclosed recipients",
                "addresses": [],
            },
        ],
        "date": datetime(1969, 2, 13, 23, 32, 54, tzinfo=timezone(-timedelta(hours=3, minutes=30))),
        "message-id": "<testabcd.1234@silly.example>",
    },
    "preamble": None,
    "content": "Testing.\n\n-- \nTaken from RFC 5322, section A.1.3.\n",
    "epilogue": None,
}

data_all = data



import pytest
from   email2dict import takes_argument

def simple(foo):
    return foo

def defaulting(foo=None):
    return foo

def kwarged(**kwargs):
    return kwargs

def arged(*foo):
    return foo

@pytest.mark.parametrize("func,arg,result", [
    (simple, "foo", True),
    (simple, "bar", False),
    (defaulting, "foo", True),
    (defaulting, "bar", False),
    (kwarged, "foo", True),
    (kwarged, "kwargs", True),
    (arged, "foo", False),
])
def test_takes_argument(func, arg, result):
    assert takes_argument(func, arg) is result



from copy import deepcopy
from datetime import datetime, timezone

data = {
    "unixfrom": "From sender@example.nil Sat Aug 18 03:12:23 2018",
    "headers": {
        "delivered-to": [
            "recipient@redacted.nil",
        ],
        "date": datetime(2018, 8, 18, 3, 11, 52, tzinfo=timezone.utc),
        "from": [
            {
                "display_name": "Steven E'Nder",
                "address": "sender@example.nil",
            },
        ],
        "to": [
            {
                "display_name": "",
                "address": "recipient@redacted.nil",
            },
        ],
        "subject": "Fwd: Your confirmed booking",
        "message-id": "<20180818031152.GA7082@example.nil>",
        "content-type": {
            "content_type": "multipart/mixed",
            "params": {},
        },
        "content-disposition": {
            "disposition": "inline",
            "params": {},
        },
        "user-agent": [
            "Mutt/1.5.24 (2015-08-30)",
        ],
    },
    "preamble": "",
    "content": [
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "text/plain",
                    "params": {},
                },
                "content-disposition": {
                    "disposition": "inline",
                    "params": {},
                },
            },
            "preamble": None,
            "content": "Here's that e-mail you wanted:\n\n",
            "epilogue": None,
        },
        {
            "unixfrom": None,
            "headers": {
                "content-type": {
                    "content_type": "message/rfc822",
                    "params": {},
                },
                "content-disposition": {
                    "disposition": "inline",
                    "params": {},
                },
            },
            "preamble": None,
            "content": {
                "unixfrom": None,
                "headers": {
                    "delivered-to": [
                        "sender@example.nil",
                    ],
                    "date": datetime(2018, 8, 18, 3, 8, 20, tzinfo=timezone.utc),
                    "sender": {
                        "display_name": "",
                        "address": "hi.booker.com@company.out.foobar.qq",
                    },
                    "message-id": "<abcdefghijklmnopqrstuv@smtpd.sendgrid.net>",
                    "to": [
                        {
                            "display_name": "",
                            "address": "sender@example.nil",
                        },
                    ],
                    "from": [
                        {
                            "display_name": "Booking Company",
                            "address": "hi.booker.com@company.out.foobar.qq",
                        },
                    ],
                    "subject": "Your confirmed booking",
                    "content-type": {
                        "content_type": "text/plain",
                        "params": {},
                    },
                },
                "preamble": None,
                "content": "We have great news! Your booking has been finalized and confirmed. Enjoy your trip, and please let us know if we can be helpful in any way. \nConfirmation ABC123 \nFlight from Newark to San Francisco EWR → SFO \nAirline PanAm \nFlight Number(s) PA 1234 \nDeparture August 19, 2018 at 3:50 PM \nArrival August 19, 2018 at 7:10 PM \nLayovers Nonstop \nClass of Service Economy \nFlight from San Francisco to Newark SFO → EWR \nAirline PanAm \nFlight Number(s) PA 4321 \nDeparture August 20, 2018 at 9:30 PM \nArrival August 21, 2018 at 6:00 AM \nLayovers Nonstop \nClass of Service Economy \nTravelers Steven E'Nder \n\nThis email is a booking confirmation, not a receipt. If you used your own payment method for this purchase, you will receive a separate email receipt after you are charged. \n",
                "epilogue": None,
            },
            "epilogue": None,
        },
    ],
    "epilogue": "\n",
}

data_all = deepcopy(data)
data_all["headers"]["mime-version"] = "1.0"
data_all["headers"]["content-type"]["params"]["boundary"] = "cWoXeonUoKmBZSoM"
data_all["content"][0]["headers"]["content-type"]["params"]["charset"] = "us-ascii"
data_all["content"][1]["content"]["headers"]["mime-version"] = "1.0"
data_all["content"][1]["content"]["headers"]["content-type"]["params"]["charset"] = "utf-8"
data_all["content"][1]["content"]["headers"]["content-transfer-encoding"] = "quoted-printable"



from datetime import datetime, timedelta, timezone

data = {
    "unixfrom": None,
    "headers": {
        "from": [
            {
                "display_name": "Mary Smith",
                "address": "mary@example.net",
            },
        ],
        "to": [
            {
                "display_name": "John Doe",
                "address": "jdoe@machine.example",
            },
        ],
        "reply-to": [
            {
                "display_name": "Mary Smith: Personal Account",
                "address": "smith@home.example",
            },
        ],
        "subject": "Re: Saying Hello",
        "date": datetime(1997, 11, 21, 10, 1, 10, tzinfo=timezone(timedelta(hours=-6))),
        "message-id": "<3456@example.net>",
        "in-reply-to": [
            "<1234@local.machine.example>",
        ],
        "references": [
            "<1234@local.machine.example>",
        ],
    },
    "preamble": None,
    "content": "This is a reply to your hello.\n\n-- \nTaken from RFC 5322, section A.2.\n",
    "epilogue": None,
}

data_all = data
